#ifndef __GENERATOR_HH__
#define __GENERATOR_HH__

/**
   \mainpage MicroJets documentation

   This package provides code for carrying out calculations of the
   properties of jets in limit of a small jet radius R.
   
   <h2>
   Essentials
   </h2>
   The usage of MicroJets is defined primarily in the following parts </p>
   - AllOrderGenerator: Base class for the calculation of resummed
     observables, available in two different flavours,
     AllOrderGeneratorRecursive and AllOrderGeneratorFlat.
   - FixedOrderGenerator: For calculating the coefficients of the
     fixed order expansion in \f$t\f$.
   - Observable.hh: Contains the definition of basic observables.

   Take a look at example.cc to get started. Available settings
   are highlighted in example2.cc, example3.cc and example4.cc.

   <h2>
   Main Features
   </h2>
   A short introduction to some of the most important features of MicroJets:
   - User input can be handled with the Input class.
   - Writing results in a file or in the terminal can be done via the Output class.
   - Calculating the average over a group of events of an observable can
     be done with average_over_group().
   - To calculate histograms, one should use the NLOHist and add_group_to_histogram() tools.
   - Filtering and trimming of events can be achieved with Filter and Trim.
   
*/

#include <vector>
#include "GSLRandom.hh"

//----------------------------------------------------------------------
/// Class to keep track of the flavour and momentum of a parton
class Parton{
public:
  enum Flavour {g,q};
  Parton():_x(1.0),_flav(q) {}
  Parton(const Flavour flav):_x(1.0),_flav(flav) {}
  Parton(const double x, const Flavour flav):_x(x),_flav(flav) {}
  Parton(const Parton& p):_x(p.x()),_flav(p.flav()) {}
  /// return x
  double x() const {return _x;}
  /// set x
  void x(const double x) {_x=x;}
  /// Scale the momentum of the parton by a factor z
  void scale(const double z) {_x*=z;}
  /// return parton flavour
  Flavour flav() const {return _flav;}
  /// set parton flavour
  void flav(const Flavour flav) {_flav=flav;}
private:
  double _x;
  Flavour _flav;
};

/// Collection of partons corresponding to an event
class Event {
public:
  typedef std::vector<Parton>::const_iterator cit;
  Event(double weight) : _weight(weight), _ihardest(-1), _xmax(-1.) {}
  Event(std::vector<Parton> partons, double weight)
    : _partons(partons), _weight(weight), _ihardest(-1), _xmax(-1.) {}

  const std::vector<Parton> & partons() const {return _partons;}

  /// return partons in the event
  std::vector<Parton> & partons() {
    _ihardest = -1; _xmax = -1.; 
    return _partons; 
  }
  
  /// add parton
  void push_back(const Parton& parton) {
    _ihardest = -1;
    _xmax = -1.;
    _partons.push_back(parton);
  }

  /// remove all partons
  void clear() {
    _ihardest = -1;
    _xmax = -1.;
    _partons.clear();
  }

  /// return number of partons
  unsigned int size() const  {return _partons.size();}

  /// access a specific parton in the event
  Parton & operator[](const int i) {
    _ihardest = -1;
    _xmax = -1.;
    return _partons[i];
  }

  const Parton & operator[](const int i) const {
    return _partons[i];
  }

  /// return weight of the event
  double weight() const {return _weight;}

  /// return a (const) reference to the hardest parton in this event
  const Parton & hardest() const {
    if (_ihardest < 0) {

      // create temporary ihardest variable (thread local) 
      int ihardest = -1;
      double xma = -1.0;
      for (unsigned i = 0; i < _partons.size(); i++) {
        double x = _partons[i].x();
        if (x > xma) {
          xma = x;
          ihardest = i;
        }
      }
      // update the class-wide _ihardest variable once we
      // we have the final value for ihardest
      _ihardest = ihardest;
      _xmax = xma;
    }

    return _partons[_ihardest];
  }

  /// return x value of hardest parton
  double xmax() const {
    if (_xmax < 0.) {
      double x=-1.;
      for(cit it=_partons.begin();it!=_partons.end();++it)
	if(it->x() > x) x=it->x();
      _xmax = x;
    }    
    return _xmax;
  }

  /// return flavour of hardest parton
  bool flav_hardest(Parton::Flavour flav) const {
    bool flav_hardest=true;
    double x=-1.;
    Parton::Flavour f=flav;
    for(cit it=_partons.begin();it!=_partons.end();++it)
      if(it->x() > x) {
	x=it->x();
	f=it->flav();
      }
    if (f!=flav) flav_hardest=false;
    return flav_hardest;
  }

  /// return delta pt after filtering the nfilt hardest subjets
  double filter(int nfilt) const {
    double sumpt=0., x=xmax();
    int n=_partons.size();
    if (nfilt < n) n=nfilt;
    for(int i=0;i<n;++i) {
      sumpt+=x;
      x=x_below(x);
    }
    return sumpt;
  }

  /// return delta pt after trimming with a treshold fcut
  double trim(double fcut) const {
    double sumpt=0.;
    for(cit it=_partons.begin();it!=_partons.end(); ++it){
      if(it->x() >= fcut) sumpt+=(it->x());
    }
    // std::cout << sumpt << " " << sumpt-1. << std::endl;
    return sumpt;
  }
protected:
  std::vector<Parton> _partons;
  double _weight;

  /// these variables are mutable, because they are used for caching purposes
  /// within the const member function hardest()
  mutable int _ihardest;
  mutable double _xmax;


  /// find momentum of parton in _partons with highest value of momentum < xma
  double x_below(double xma) const {
    double x=-1.;
    for(cit it=_partons.begin();it!=_partons.end();++it)
      if((it->x() > x) && (it->x() < xma))
	x=it->x();
    return x;
  }
};

//----------------------------------------------------------------------
/// Abstract base class that defines the interface for generators of
/// microjet ensembles
class Generator {
public:
  Generator(int seed,Parton::Flavour flav) : _rng(seed),_flav(flav) {
    _CA=3.;_CF=4./3.;_TR=1./2.;_nf=5.;
  }
  Generator(int seed, Parton::Flavour flav, double ca, double cf, double tr, double nf)
    : _rng(seed),_flav(flav),_CA(ca),_CF(cf),_TR(tr),_nf(nf) {}

  virtual ~Generator() {}

  /// compute the next event group
  virtual void next_event_group() =0;

  /// set tmax
  virtual void set_tmax(double) =0;
  /// set tmin (starting value)
  virtual void set_tmin(double) =0;

  /// reset event group
  virtual void reset() =0;
  
  /// return current event group
  const std::vector<Event> & events() const {return _events;}

protected:
  std::vector<Event> _events;
  GSLRandom _rng;
  Parton::Flavour _flav;
  double _CA,_CF,_TR,_nf;
};

#endif // __GENERATOR_HH__
