#ifndef __INPUT_HH__
#define __INPUT_HH__

#include <stdlib.h>
#include <string>
#include <iostream>
#include <vector>
#include "CmdLine.hh"
#include "Generator.hh"



/// Class to handle of user input.
/// Options, with default values, are :\n
/// [ -q | -g ] [ -seed 1 ] \n
/// [ -fixed-order [-order 1] | -all-order [-non-recursive] [-t 0.1]] \n
/// [ -nev 1e3 ] [ -eps 1e-3 ] [ -dt 0.1 ] [ -fcut 0.05 ] [ -dfcut 0.005 ] \n
/// [ -nsteps 1 ] [ -nfilt 2 ] [ -output std::cout ] [ -nbin 10 ] \n
/// [ -save-config "" | -load-config ""] \n
/// [ -ca 3 ] [ -cf 4/3 ] [ -tr 1/2 ] [ -nf 5 ]
class Input{
public:
    Input(int argc, char ** argv)
      : _cmdline(argc, argv) {
      // seed for rng
      seed=_cmdline.value("-seed",1);
      // constants in splitting fcts
      ca=_cmdline.value("-ca",3.);
      cf=_cmdline.value("-cf",4./3.);
      tr=_cmdline.value("-tr",1./2.);
      nf=_cmdline.value("-nf",5.);
      // parameters
      order = _cmdline.value("-order",1);
      nev=_cmdline.value("-nev",1e3);
      eps=_cmdline.value("-eps",1e-3);
      t=_cmdline.value("-t",0.1);
      dt=_cmdline.value("-dt",0.1);
      nsteps=_cmdline.value("-nsteps",1);
      fcut=_cmdline.value("-fcut",0.05);
      dfcut=_cmdline.value("-dfcut",0.005);
      nfilt=_cmdline.value("-nfilt",2);
      nbin=_cmdline.value("-nbin",10);

      // Flavour of initiating parton : q or g
      flav=Parton::q;
      if (_cmdline.present("-g")) flav=Parton::g;
      if (_cmdline.present("-q")) flav=Parton::q;

      // Output parameters
      filename=_cmdline.value<std::string>("-output","");
      saveconfig=false;
      configname=_cmdline.value<std::string>("-save-config","");
      if (configname!="") saveconfig=true;
      std::string temp=_cmdline.value<std::string>("-load-config","");
      if (temp!="") {
	loadconfig=true;
	configname=temp;
	saveconfig=false;
      }
      // Generator settings
      fixedorder=false;
      if (_cmdline.present("-fixed-order")) fixedorder=true;
      if (_cmdline.present("-all-order")) fixedorder=false;
      nonrecursive=false;
      if (_cmdline.present("-non-recursive")) nonrecursive=true;
      
      command=_cmdline.command_line();
      time=_cmdline.time_stamp();

    }
 
  /// If fixedorder is set to true, use FixedOrderGenerator, else use AllOrderGenerator
  bool fixedorder;
  /// If nonrecursive is true use AllOrderGeneratorFlat, else use AllOrderGeneratorRecursive
  bool nonrecursive;
  /// Save particle configuration
  bool saveconfig;
  /// Load a particle configuration
  bool loadconfig;
  /// Number of events
  double nev;
  /// Epsilon value for integral limits
  double eps;
  /// Starting value of angular scale
  double t;
  /// Step size for t
  double dt;
  /// Treshold momentum for trimming
  double fcut;
  /// Step size for fcut
  double dfcut;
  /// Value of CA used in splitting functions
  double ca;
  /// Value of CF in splitting functions
  double cf;
  /// Value of TR in splitting functions
  double tr;
  /// Number of fermions in splitting functions
  double nf;
  /// Seed to use for the Random Number Generator
  int seed;
  /// Order to compute with FixedOrderGenerator
  int order;
  /// Number of steps
  int nsteps;
  /// Number of hardest subjets to use in filtering
  int nfilt;
  /// Number of bins for histograms
  int nbin;
  /// Flavour of the initial parton
  Parton::Flavour flav;
  /// String with file name for the output file
  std::string filename;
  /// String with file name for the configuration file
  std::string configname;
  /// String with input command
  std::string command;
  /// String with time stamp
  std::string time;

  /// return value defined by string
  double cmdline(string s, double def_val) {
    return _cmdline.value(s,def_val);
  }

  /// return bool defined by string
  bool cmdline(string s) {
    return _cmdline.present(s);
  }

  /// assert that all options are used
  void assert_all_options_used() {
    _cmdline.assert_all_options_used();
  }
  
  /// Read next event group from input stream
  std::vector<Event> next_event_group(std::istream& in) {
    std::vector<Parton> state;
    std::vector<Event> events;
    bool ingroup=true;
    while(ingroup) {
      std::string line;
      std::getline(in,line);
      if (line.substr(0,8)=="</group>")
	ingroup=false;
      else if (line.substr(0,9)=="<event w=") {
	double weight=atof(line.substr(9,line.size()-10).c_str());
	bool inevent=true;
	state.clear();
	while(inevent) {
	  std::getline(in,line);
	  if (line.substr(0,8)=="</event>") inevent=false;
	  else {
	    double z=atof(line.substr(2).c_str());
	    if (line.substr(0,1)=="q")
	      state.push_back(Parton(z,Parton::q));
	    if (line.substr(0,1)=="g")
	      state.push_back(Parton(z,Parton::g));
	  }
	}
	events.push_back(Event(state,weight));
      }
    }
    return events;
  }

  /// Checks wether the end of the file was reached
  /// return false if there is a new group
  bool file_end(std::istream& in) {
    std::string line;
    std::getline(in,line);
    if (line.substr(0,7)=="<group>")
      return false;
    return true;
  }

private:
  CmdLine _cmdline;
};

#endif // __INPUT_HH__
