#ifndef __OBSERVABLE_HH__
#define __OBSERVABLE_HH__
#include "Generator.hh"
#include "NLOHist.hh"
#include "AverageAndError.hh"
#include <boost/utility/enable_if.hpp>
/// \file Observable.hh Contains definitions of some basic observables.
/// For averages use average_over_group(events, function). 
/// For histograms use add_group_to_histogram(hist, events, function)

/// return pt of hardest parton in event
double xmax(const Event & ev) {return ev.xmax();}

/// return delta pt of hardest parton in event
double deltaxmax(const Event & ev) {return (ev.xmax()-1.);}

/// return log pt of hardest parton in event
double logxmax(const Event & ev) {return log(ev.xmax());}

/// return log^2 pt of hardest parton in event
double log2xmax(const Event & ev) {return pow(log(ev.xmax()),2);}

/// return pt^6 of hardest parton in event
double xmax6(const Event & ev) {return pow(ev.xmax(),6);}

/// return pt^n of hardest parton in event
double xmaxpow(const Event & ev, const double npow) {return pow(ev.xmax(),npow);}

/// Fill histogram with parton momentum in event
void xinclusive(NLOHist & hist, const Event& ev) {
  int n=ev.size();
  for(int i=0; i<n; ++i) {
    hist.add_entry(ev[i].x(),ev.weight());
  }
}

/// Fill histogram with parton momentum^n in event
void xinclpow(NLOHist & hist, const Event& ev, const double npow) {
  int n=ev.size();
  for(int i=0; i<n; ++i) {
    hist.add_entry(pow(ev[i].x(),npow),ev.weight());
  }
}

/// Calculate sum of parton momentum^n in event
double xinclpow(const Event& ev, const double npow) {
  int n=ev.size();
  double value=0.;
  for(int i=0; i<n; ++i) {
    value+=pow(ev[i].x(),npow);
  }
  return value;
}

/// Function to calculate the probability of finding a
/// hardest parton of a given flavour. 
/// Returns 1 if hardest parton has flavour flav, 0 if not
class FlavHardest {
public:
  FlavHardest(Parton::Flavour flav) : _flav(flav) {}
  double operator()(const Event & ev) const {
    return ev.flav_hardest(_flav);
  }
private:
  Parton::Flavour _flav;
};

/// Filtering of event with nfilt hardest subjets
class Filter {
public:
  Filter(int nfilt) : _nfilt(nfilt) {}
  /// return delta pt of nfilt hardest subjets
  double operator()(const Event & ev) const {
    return ev.filter(_nfilt);
  }
private:
  int _nfilt;
};

/// Triming of event with treshold momentum fcut.
class Trim {
public:
  Trim(double fcut) : _fcut(fcut) {}
  /// return delta pt of trimmed event
  double operator()(const Event & ev) const {
    return ev.trim(_fcut);
  }
  double fcut() {return _fcut;}
  void set_fcut(double fcut) {_fcut=fcut;}
  void operator+=(double dfcut) {_fcut+=dfcut;}
  
private:
  double _fcut;
};


/// nth power of hardest parton momentum of an event 
class Xmaxpow {
public:
  Xmaxpow(double n) : _npow(n) {}
  /// return pt^n of hardest parton in event
  double operator()(const Event & ev) const {
    return xmaxpow(ev,_npow);
  }
private:
  double _npow;
};

/// nth power of parton momentum of an event 
class Xinclpow {
public:
  Xinclpow(double n) : _npow(n) {}
  /// fill hist with pt^n of partons in event
  void operator()(NLOHist & hist, const Event& ev) const {
    xinclpow(hist,ev,_npow);
  }
  /// Calculate the sum over an event of xinclpow
  double operator()(const Event& ev) const {
    return xinclpow(ev,_npow);
  }
private:
  double _npow;
};

typedef std::vector<Event>::const_iterator CstEvIt;


/// Calculate the average over a group of events of an observable f
template<class T> 
double average_over_group(const std::vector<Event> & events, const T & f) {
  double value=0.;
  for(CstEvIt it=events.begin();it!=events.end();++it)
    value += f(*it)*(it->weight());
  return value;
}

/// For each event, fill the histogram with the output of the function
/// f applied to that event.
template<class T> 
void add_group_to_histogram(NLOHist& hist, const std::vector<Event> & events, const T & f) {
  for(CstEvIt it=events.begin(); it!=events.end(); ++it) {
    hist.add_entry(f(*it), it->weight());
  }
}

/// Template specialization for the case of inclusive observables
template<>
void add_group_to_histogram<void(NLOHist&,const Event&)>
(NLOHist& hist,const std::vector<Event> & events,void(&f)(NLOHist&,const Event&)){
  for(CstEvIt it=events.begin(); it!=events.end(); ++it) {
    f(hist,*it);
  }
}

/// Function overload for the case of inclusive observables with external parameters
void add_group_to_histogram
(NLOHist& hist,const std::vector<Event> & events, const Xinclpow & f){
  for(CstEvIt it=events.begin(); it!=events.end(); ++it) {
    f(hist,*it);
  }
}

#endif // __OBSERVABLE_HH__
