!----------------------------------------------------------------------
!! A module to compute a partonic spectrum, either from LO matrix elements,
!! from an input file, or using a simple toy model

module parton_spect
  use common_setup
  use hoppet_v1, EvolvePDF_hoppet => EvolvePDF, InitPDF_hoppet => InitPDF
  use sub_defs_io

  implicit none
  integer :: spectrum_type
  real(dp), pointer :: parton_spectrum(:,:)
  real(dp),external :: quarkspectrum, gluonspectrum

  !! taking a specific channel
  logical*1, private :: gg2qq, gg2gg, qq2gg, qq2qq, qg2qg
  real(dp), private  :: bin_ymin, bin_ymax
  real(dp), private  :: rts, ptmin
  
contains
  subroutine setup_partonspec (pts, rts_init, ptmin_init, xmur, xmuf)
    real(dp), intent(in) :: rts_init, ptmin_init, xmur, xmuf
    real(dp), pointer    :: pts(:)
    character(len=180)   :: in_file, save_file, pdf_name
    real(dp)             :: pt_dummy
    integer              :: ii, ipt, isave

    ptmin = ptmin_init
    rts   = rts_init
    
    ! allocate and set up the parton spectrum (0=gluon, 1=all quarks)
    ! these are dsigma/dlnpt; ultimately we'll be aiming to calculate
    ! this information from scratch (or at least read it in from a file)
    call AllocGridQuant(grid, parton_spectrum, 0,1)

    bin_ymin = dble_val_opt("-bin-ymin", 0.1_dp)
    bin_ymax  = dble_val_opt("-bin-ymax" , -2.0_dp)

    gg2qq = .true.
    gg2gg = .true.
    qq2gg = .true.
    qq2qq = .true.
    qg2qg = .true. 
    if (log_val_opt("-gg2qq").or.log_val_opt("-gg2gg").or.log_val_opt("-qq2gg")&
         &.or.log_val_opt("-qq2qq").or.log_val_opt("-qg2qg")) then
       gg2qq = .false.
       gg2gg = .false.
       qq2gg = .false.
       qq2qq = .false.
       qg2qg = .false. 
    endif
    if (log_val_opt("-gg2qq")) then
       gg2qq = .true.
    endif
    if (log_val_opt("-gg2gg")) then
       gg2gg = .true.
    endif
    if (log_val_opt("-qq2gg")) then
       qq2gg = .true.
    endif
    if (log_val_opt("-qq2qq")) then
       qq2qq = .true.
    endif
    if (log_val_opt("-qg2qg")) then
       qg2qg = .true.
    endif

    if (log_val_opt("-in")) then
       spectrum_type = 1
       ! read spectrum from input card, e.g.,
       ! -in ../output/spectrum-lo-code.dat : use spectrum produced with LO code
       ! doesn't read pt, so spectrum has to be correctly binned.
       in_file = string_val_opt("-in")
       write(*,*) 'Using input spectrum ',in_file
       write(*,*) 'Warning: the pt values have to be correct'
       open(unit=70, status="unknown", file=trim(in_file))         
       ii=0
       do
          read(70,'(f15.5,2es15.7)',end=888) pt_dummy, parton_spectrum(ii,0), &
               & parton_spectrum(ii,1)
          ii=ii+1
       enddo
888    continue
    elseif (log_val_opt("-lo-dummy")) then
       spectrum_type = 2
       write(*,*) 'Using non-optimal LO code (use -lo-code option instead)'
       pdf_name = string_val_opt('-pdf','CT10.LHgrid')
       write(hoppet_idev,"(a)") "# pdf = "//pdf_name
       call InitPDFsetByName(pdf_name) !use cteq66
       call InitPDF(0) !best fit pdf
       call random_seed
       do ipt = 0, grid%ny
          ! local implementation of the LO code using MC integration,
          ! only for educational purposes and cross checks
          write(*,'(1a1,a,f5.1,a,$)') char(13), ' Status: ', ipt*100.0/grid%ny, '%  '
          parton_spectrum(ipt,0) = parton_spectrum_LO(pts(ipt),0)
          parton_spectrum(ipt,1) = parton_spectrum_LO(pts(ipt),1)
          parton_spectrum(ipt,:) = parton_spectrum(ipt,:)
       enddo
    elseif (log_val_opt("-toy")) then
       spectrum_type = 3
       write(*,*) 'Using toy spectrum (simple power law)'
       ! gluon spectrum
       parton_spectrum(:,0) = (500.0_dp/pts)**3.5_dp * (one-xValues(grid))**6.0_dp
       ! quark spectrum
       parton_spectrum(:,1) = (500.0_dp/pts)**3.0_dp * (one-xValues(grid))**4.0_dp 
    else
       ! use spectrum from LO code
       pdf_name = string_val_opt('-pdf','CT10.LHgrid')
       write(hoppet_idev,"(a)") "# pdf = "//pdf_name
       call InitPDFsetByName(pdf_name) 
       call InitPDF(0) !best fit pdf
       write(*,*) 'Using incl-spec-code/libinclspec.a to create jet spectrum with pdf = '//trim(pdf_name)
       spectrum_type = 0
       call initspectrum(rts,xmur,xmuf,gg2qq,gg2gg,qq2gg,qq2qq,qg2qg)

       do ipt = 0, grid%ny
          write(*,'(1a1,a,f5.1,a,$)') char(13), ' Status: ', ipt*100.0/grid%ny, '%  '
          ! use dsigma_dpt from LOInclSpectrum/lo.cc to create spectrum
          parton_spectrum(ipt,0) = gluonspectrum(pts(ipt),bin_ymin,bin_ymax)
          parton_spectrum(ipt,1) = quarkspectrum(pts(ipt),bin_ymin,bin_ymax)
       enddo
    endif
    write(0,'(a)') '[quark and gluon spectrum have been computed]'
    if (log_val_opt("-save")) then
       ! save the parton spectrum in a file
       save_file = string_val_opt("-save")
       isave = idev_open_opt("-save")
       write(*,*) 'Saving spectrum in ',save_file    
       do ipt = 0, grid%ny
          write(isave,'(f15.5,2es15.7)') pts(ipt), parton_spectrum(ipt,:)
       enddo
    endif
    if (log_val_opt("-nologpt")) then
       ! use -nologpt to output dsigma/dpt instead of dsigma/dlogpt
       parton_spectrum(:,0) = parton_spectrum(:,0) / pts(:)
       parton_spectrum(:,1) = parton_spectrum(:,1) / pts(:)
    endif
  end subroutine setup_partonspec

  !----------------------------------------------------------------------
  real(dp) function spectrum_g(ptval)
    real(dp), intent(in) :: ptval
    real(dp) :: y
    y = log(rts/ptval)
    spectrum_g = EvalGridQuant(grid,parton_spectrum(:,0), y)

  end function spectrum_g

  real(dp) function spectrum_q(ptval)
    real(dp), intent(in) :: ptval
    real(dp) :: y
    y = log(rts/ptval)
    spectrum_q = EvalGridQuant(grid,parton_spectrum(:,1), y)

  end function spectrum_q

  !----------------------------------------------------------------------  
  ! Calculation of LO jet spectrum
  function parton_spectrum_LO (pt, parton_type)
    real(dp) :: parton_spectrum_LO
    real(dp), intent(in) :: pt
    integer, intent(in) :: parton_type
    integer,parameter :: ncall_mc=200
    integer :: i
    real(dp) :: r, maxrap, lower, upper, sum_mc

    if (pt.ge.0.99_dp*0.5_dp*rts) then
       parton_spectrum_LO = 0.0_dp
    else
       maxrap = log(rts/(two*pt) + sqrt(rts*rts/(two*pt*two*pt)-1))
       if ((bin_ymax.lt.bin_ymin).or.(bin_ymin.lt.-maxrap)) then
          lower = -maxrap
       else 
          lower = bin_ymin
       endif
       if ((bin_ymax.lt.bin_ymin).or.(bin_ymax.gt.maxrap)) then
          upper = maxrap
       else 
          upper = bin_ymax
       endif
       ! monte carlo integration
       sum_mc=0.0_dp
       do i=1,ncall_mc
          call random_number(r)
          sum_mc=sum_mc+dsigma_dydpt2(rts, pt, lower + r*(upper-lower), parton_type)
       enddo
       ! 2 pt^2 factor for dsigma/dpt^2 -> dsigma/dlnpt
       parton_spectrum_LO = (two*pt*pt)*sum_mc*(upper-lower)/ncall_mc
    endif
  end function parton_spectrum_LO

  function dsigma_dydpt2 (rts, pt, y, parton_type)
    real(dp) :: dsigma_dydpt2
    real(dp), intent(in) :: rts, pt, y
    integer, intent(in) :: parton_type
    integer,parameter :: ncall_mc=200
    integer :: i
    real(dp) :: r,ymin_integ, ymax_integ, sum_mc, alphas
    real(dp),external :: alphasPDF

    if (abs(y).gt.(log(rts/(two*pt) + sqrt(rts*rts/(two*pt*two*pt)-one)))) then
       dsigma_dydpt2 = 0.0_dp
    else
       ymin_integ = y - log(rts*exp(y)/pt - one)
       ymax_integ = y + log(rts*exp(-y)/pt - one)

       ! monte carlo integration
       sum_mc=0.0_dp
       if (parton_type.eq.1) then
          do i=1,ncall_mc
             call random_number(r)
             sum_mc=sum_mc+dsigma_dydydpt2_quark(rts, pt, y, ymin_integ &
                  & + r*(ymax_integ-ymin_integ))
          enddo
       elseif (parton_type.eq.0) then
          do i=1,ncall_mc
             call random_number(r)
             sum_mc=sum_mc+dsigma_dydydpt2_gluon(rts, pt, y, ymin_integ &
                  & + r*(ymax_integ-ymin_integ))
          enddo
       endif
       alphas=alphasPDF(1.0_dp*pt) !mu_r=1.0
       dsigma_dydpt2 = pi*alphas*alphas*sum_mc*(ymax_integ-ymin_integ)/ncall_mc
    endif

  end function dsigma_dydpt2


  function dsigma_dydydpt2_quark (rts, pt, y1, y2)
    real(dp) :: dsigma_dydydpt2_quark
    real(dp), intent(in) :: rts, pt, y1, y2
    real(dp),parameter :: gev2nb = 389379
    real(dp) :: xpdf1(-6:6), xpdf2(-6:6)
    integer :: i, j
    real(dp) :: x1, x2, costheta, s, t, u, sum_me, res_pdf

    x1=(pt/rts)*(exp(y1)+exp(y2))
    x2=(pt/rts)*(exp(-y1)+exp(-y2))
    costheta=tanh((y1-y2)/two)

    s = x1*x2*rts**2
    t = -s*(1-costheta)/two
    u = -s*(1+costheta)/two

    ! (C++ equivalent)
    ! LHAPDF::xfx(x1,1.0_dp*pt,xpdf1)
    ! LHAPDF::xfx(x2,1.0_dp*pt,xpdf2)
    call evolvePDF(x1,1.0_dp*pt,xpdf1) !1.0 = mu_f
    call evolvePDF(x2,1.0_dp*pt,xpdf2) !1.0 = mu_f
    
    sum_me = 0.0_dp

    ! qq' + qqbar' + qbarq' + qbarqbar' contribution
    res_pdf=0.0_dp 
    do i = 1,6
       do j = 1,6
          if (i.ne.j) then
             res_pdf = res_pdf + (xpdf1(i) + xpdf1(-i))*(xpdf2(j) + xpdf2(-j))
          endif
       enddo
    enddo
    if (qq2qq) then
       sum_me = sum_me + res_pdf * (Mqqp2qqp(s,t,u) + Mqqp2qqp(s,u,t))
    endif

    ! qq + qbarqbar contribution
    res_pdf=0.0_dp
    do i = 1,6
       res_pdf = res_pdf + (xpdf1(i)*xpdf2(i) + xpdf1(-i)*xpdf2(-i))
    enddo
    if (qq2qq) then
       sum_me = sum_me + res_pdf*Mqq2qq(s,t,u)
    endif

    ! qqbar + qbarq contribution
    res_pdf=0.0_dp
    do i = 1,6
       res_pdf = res_pdf + xpdf1(i)*xpdf2(-i) + xpdf1(-i)*xpdf2(i)
    enddo
    if (qq2qq) then
       sum_me = sum_me + res_pdf*(Mqqbar2qpqbarp(s,t,u) + Mqqbar2qpqbarp(s,u,t) &
            & + Mqqbar2qqbar(s,t,u) + Mqqbar2qqbar(s,u,t))
    endif

    ! qg + gq + qbarg +gqbar contribution
    res_pdf=0.0_dp
    do i = 1,6
       res_pdf = res_pdf + (xpdf1(i)+xpdf1(-i))*xpdf2(0)
    enddo
    if (qg2qg) then
       sum_me = sum_me + res_pdf*Mqg2qg(s,t,u)
    endif
    res_pdf=0.0_dp
    do i = 1,6
       res_pdf = res_pdf + xpdf1(0)*(xpdf2(i)+xpdf2(-i))
    enddo
    if (qg2qg) then
       sum_me = sum_me + res_pdf*Mqg2qg(s,u,t)
    endif

    ! gg contribution
    res_pdf = xpdf1(0)*xpdf2(0)
    if (gg2qq) then
       sum_me = sum_me + res_pdf*(Mgg2qqbar(s,t,u) + Mgg2qqbar(s,u,t))
    endif

    dsigma_dydydpt2_quark = gev2nb*sum_me/(s**2)

  end function dsigma_dydydpt2_quark

  function dsigma_dydydpt2_gluon (rts, pt, y1, y2)
    real(dp) :: dsigma_dydydpt2_gluon
    real(dp), intent(in) :: rts, pt, y1, y2
    real(dp),parameter :: gev2nb = 389379
    real(dp) :: xpdf1(-6:6), xpdf2(-6:6)
    integer :: i
    real(dp) :: x1, x2, costheta, s, t, u, sum_me, res_pdf

    x1=(pt/rts)*(exp(y1)+exp(y2))
    x2=(pt/rts)*(exp(-y1)+exp(-y2))
    costheta=tanh((y1-y2)/two)

    s = x1*x2*rts**2
    t = -s*(1-costheta)/two
    u = -s*(1+costheta)/two

    ! (C++ equivalent)
    ! LHAPDF::xfx(x1,1.0_dp*pt,xpdf1)
    ! LHAPDF::xfx(x2,1.0_dp*pt,xpdf2)
    call evolvePDF(x1,1.0_dp*pt,xpdf1) !1.0 = mu_f
    call evolvePDF(x2,1.0_dp*pt,xpdf2) !1.0 = mu_f

    sum_me = 0.0_dp

    ! qqbar + qbarq contribution
    res_pdf=0.0_dp
    do i=1,6
       res_pdf = res_pdf + xpdf1(i)*xpdf2(-i) + xpdf1(-i)*xpdf2(i)
    enddo
    if (qq2gg) then
       sum_me = sum_me + res_pdf*(Mqqbar2gg(s,t,u))
    endif

    ! qg + gq + qbarg +gqbar contribution
    res_pdf=0.0_dp
    do i = 1,6
       res_pdf = res_pdf + (xpdf1(i)+xpdf1(-i))*xpdf2(0)
    enddo
    if (qg2qg) then
       sum_me = sum_me + res_pdf*Mqg2qg(s,u,t)
    endif
    res_pdf=0.0_dp
    do i = 1,6
       res_pdf = res_pdf + xpdf1(0)*(xpdf2(i)+xpdf2(-i))
    enddo
    if (qg2qg) then
       sum_me = sum_me + res_pdf*Mqg2qg(s,t,u)
    endif

    ! gg contribution
    res_pdf = xpdf1(0)*xpdf2(0)
    if (gg2gg) then
       sum_me = sum_me + res_pdf*Mgg2gg(s,t,u)
    endif

    dsigma_dydydpt2_gluon = gev2nb*sum_me/(s**2)

  end function dsigma_dydydpt2_gluon
  
  !----------------------------------------------------------------------
  ! LO matrix elements

  ! gg -> gg
  real(dp) function Mgg2gg(s, t, u)
    real(dp), intent(in) :: s, t, u
    Mgg2gg = (9.0_dp/2.0_dp)*(3.0_dp-(u*t)/(s*s)-(u*s)/(t*t)-(s*t)/(u*u))
  end function Mgg2gg

  ! gg -> qqbar
  real(dp) function Mgg2qqbar (s, t, u)
    real(dp), intent(in) :: s, t, u
    Mgg2qqbar = hoppet_nf*((1.0_dp/6.0_dp)*(u/t+t/u) &
         & - (3.0_dp/8.0_dp)*(u*u+t*t)/(s*s))
  end function Mgg2qqbar

  ! qg -> qg
  real(dp) function Mqg2qg (s, t, u)
    real(dp), intent(in) :: s, t, u
    Mqg2qg = (u*u+s*s)/(t*t) - (4.0_dp/9.0_dp)*(u/s+s/u)
  end function Mqg2qg

  ! qq' -> qq'
  real(dp) function Mqqp2qqp (s, t, u)
    real(dp), intent(in) :: s, t, u
    Mqqp2qqp = (4.0_dp/9.0_dp)*(s*s+u*u)/(t*t)
  end function Mqqp2qqp

  ! qqbar -> q'qbar'
  real(dp) function Mqqbar2qpqbarp (s, t, u)
    real(dp), intent(in) :: s, t, u
    Mqqbar2qpqbarp = (hoppet_nf-1)*(4.0_dp/9.0_dp)*(t*t+u*u)/(s*s)  
  end function Mqqbar2qpqbarp

  ! qq -> qq
  real(dp) function Mqq2qq (s, t, u)
    real(dp), intent(in) :: s, t, u
    Mqq2qq = ((4.0_dp/9.0_dp)*((s*s+u*u)/(t*t)+(s*s+t*t)/(u*u)) &
         & - (8.0_dp/27.0_dp)*(s*s)/(t*u))
  end function Mqq2qq

  ! qqbar -> qqbar
  real(dp) function Mqqbar2qqbar (s, t, u)
    real(dp), intent(in) :: s, t, u
    Mqqbar2qqbar = (4.0_dp/9.0_dp)*((t*t+u*u)/(s*s)+(s*s+u*u)/(t*t)) &
         & - (8.0_dp/27.0_dp)*(u*u)/(s*t)
  end function Mqqbar2qqbar

  ! qqbar -> gg
  real(dp) function Mqqbar2gg (s, t, u)
    real(dp), intent(in) :: s, t, u
    Mqqbar2gg = ((32.0_dp/27.0_dp)*(u/t+t/u) &
         & - (8.0_dp/3.0_dp)*(t*t+u*u)/(s*s))
  end function Mqqbar2gg
  
end module parton_spect
