
/// \file example.cc Simple example, showing calculation of a small set of observables
///
/// Usage:
///
/// \code
///     ./example [ -fixed-order [ -order O ] | -all-order [ -non-recursive ] [ -t T ] ] 
///               [ -q | -g ] [ -seed S ] [ -nev N ] [ -eps E ] [ -nbin B ]   
/// \endcode
///
/// The code can be run to give fixed order results (-fixed-order),
/// specifically the coefficient of t^O, or all-order results
/// (-all-order) for a given value of t=T.
///
/// You can specify whether you start with a quark (default) or a
/// gluon with the -q and -g options.
///
/// Technical parameters:
///
/// - -non-recursive specifies an alternative all-order implementation
/// - -nev N         indicates that the program should generate N events (default 10^4)
/// - -eps E         specifies the value of the cutoff: E < z < 1-E.
/// - -seed S        specifies the random seed (default 1)
/// - -nbin B        indicates the number of bins for histograms
///

#include "CmdLine.hh"
#include "Observable.hh"
#include "NLOHist.hh"
#include "AverageAndError.hh"
#include "Generator.hh"
#include "AllOrderGeneratorFlat.hh"
#include "FixedOrderGenerator.hh"
#include "AllOrderGeneratorRecursive.hh"

using namespace std;

/// Basic example program to illustrate the use of the small-R Monte Carlo generator
int main(int argc, char ** argv) {
  CmdLine cmdline(argc,argv);
  int seed = cmdline.value("-seed",1);
  Parton::Flavour flav = Parton::q;
  if (cmdline.present("-q")) flav = Parton::q;
  if (cmdline.present("-g")) flav = Parton::g;
  int order = cmdline.value("-order",1);
  double nev = cmdline.value("-nev",1e4);
  // eps is the epsilon value for integral limits in all order code
  double eps = cmdline.value("-eps",1e-3);
  double t = cmdline.value("-t",0.1);
  int nbin = cmdline.value("-nbin",10);
  Generator * generator;
  if (cmdline.present("-fixed-order") && !cmdline.present("-all-order")) {
    generator = new FixedOrderGenerator(seed,flav,order);
  }
  else if (cmdline.present("-non-recursive")) {
    generator = new AllOrderGeneratorFlat(seed,flav,eps,t);
  }
  else {
    generator = new AllOrderGeneratorRecursive(seed,flav,eps,t);
  }
  
  AverageAndError deltapt,logpt;
  NLOHist histo_xmax("x max",0.,1.+1e-9,nbin);
  //
  for (int iev = 0; iev < nev; iev++) {
    // events may come in "groups" (like for a NLO program);
    // generate the next group
    generator->next_event_group();

    // get a reference to the next group of events
    const vector<Event> & events = generator->events();

    // Calculate observables: deltaxmax logxmax and xmax are all
    // functions of an event, defined in Observable.hh .
    // We keep track of the averages of deltaxmax and logxmax
    deltapt += average_over_group(events,deltaxmax);
    logpt   += average_over_group(events,logxmax);
    // and we histogram the xmax variable
    add_group_to_histogram(histo_xmax, events, xmax);

    // Reset to initial configuration for next event
    histo_xmax.collate_event();
    generator->reset();
  }
  cout.precision(15);
  cout << "average delta_pt is " << deltapt.average() <<" +- " << deltapt.error()<< endl;
  cout << "average log_pt is "   << logpt.average()   <<" +- " << logpt.error()<< endl;
  cout << "# histogram of x max" << endl;
  histo_xmax.write(cout,0);
}

