/// \file example4.cc Example that prints out the trimmed delta pt over a range
///  of values for fcut.
///
/// Usage:
/// 
/// \code
///     ./example4 [ -fixed-order [ -order O ] | -all-order [ -non-recursive ] [ -t T ] ]
///                [ -q | -g ] [ -seed S ] [ -nev N ] [ -eps E ] [ -nbin B ]
///                [ -fcut F ] [ -nfilt M ] [ -nsteps NS ] [ -dfcut DF ] [ -output FN ] 
///                [ -ca CA ] [ -cf CF ] [ -tr TR] [ -nf NF ]
///               
/// \endcode
///
/// The code can be run to give fixed order results (-fixed-order),
/// specifically the coefficient of t^O, or all-order results
/// (-all-order) for a given value of t=T.
///
/// You can specify whether you start with a quark (default) or a
/// gluon with the -q and -g options.
///
/// Technical parameters (see default values in Input class):
///
/// - -non-recursive specifies an alternative all-order implementation
/// - -nev N         indicates that the program should generate N events (default 10^4)
/// - -eps E         specifies the value of the cutoff: E < z < 1-E.
/// - -seed S        specifies the random seed (default 1)
/// - -nbin B        indicates the number of bins for histograms
/// - -fcut F        specifies the treshold value F < z for trimming
/// - -nfilt M       specifies the number of hardest subjets for filtering
/// - -nsteps NS     specifies the number of values fcut to compute
/// - -dfcut DF      specifies the increment in fcut at each step
/// - -output FN     indicates the name of output file (default is to std::cout)
/// - -ca CA         specifies the value of CA in pgg
/// - -cf CF         specifies the value of CF in pqq
/// - -tr TR         specifies the value of TR in pqg
/// - -nf NF         specifies the value of NF in pqg
///

#include "Input.hh"
#include "NLOHist.hh"
#include "AverageAndError.hh"
#include "Observable.hh"
#include "Output.hh"
#include "Generator.hh"
#include "AllOrderGeneratorFlat.hh"
#include "FixedOrderGenerator.hh"
#include "AllOrderGeneratorRecursive.hh"
#include <sstream>

using namespace std;

int main(int argc, char ** argv) {
  Input in(argc,argv);
  in.assert_all_options_used();
  cout << "# " << in.command << "\n# " << in.time << endl;
  Output output(in.filename,in.command,in.time);

  Generator * generator;
  if (in.fixedorder) {
    generator = new FixedOrderGenerator(in.seed,in.flav,in.order,
					in.ca,in.cf,in.tr,in.nf);
  }
  else if (in.nonrecursive) {
    generator = new AllOrderGeneratorRecursive(in.seed,in.flav,in.eps,in.t,
					       in.ca,in.cf,in.tr,in.nf);
  }
  else {
    generator = new AllOrderGeneratorFlat(in.seed,in.flav,in.eps,in.t,
					  in.ca,in.cf,in.tr,in.nf);
  }
  
  vector< map<string,AverageAndError> > averages(in.nsteps);
  // vector< map<string,SimpleHist> > histos(in.nsteps);
  vector< vector<NLOHist> > histos(in.nsteps);

  Trim trim(in.fcut);
  // Loop over events
  for (int iev = 0; iev < in.nev; iev++) {
    // events may come in "groups" (like for a NLO program);
    // generate the next group
    generator->next_event_group();
    // Loop from fcut=in.fcut to in.fcut+(in.nsteps-1)*in.dfcut
    for (int k=0;k<in.nsteps;++k) {
      // get a reference to the next group of events
      const vector<Event> & events = generator->events();
      // calculate observables
       ostringstream oss;
       oss << "trimmed z [fcut=" << trim.fcut() << "]";
       averages[k][oss.str()]+= average_over_group(events,trim);
      trim+=in.dfcut;
    }

    // Write results when writing_step(iev)==true
    if(output.writing_step(iev)) {
      output.write(averages,histos,iev,in.fcut,in.dfcut,
		   in.nsteps,string(""));
      cout << "# Currently at " << iev << " out of "
		<< in.nev << " events." << endl;
    }
    
    // Reset to initial configuration for next event and put fcut to initial value
    trim.set_fcut(in.fcut);
    generator->reset();
  }
  output.write(averages,histos,in.nev,in.fcut,in.dfcut,
	       in.nsteps,string(""));
  cout << "# Simulation completed" << endl;
}

